/*
 * -------------------------------------------
 *    MSP432 DriverLib - v3_50_00_02
 * -------------------------------------------
 *
 * --COPYRIGHT--,BSD,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
/*******************************************************************************
 * MSP432 Flash Controller - Programming Calibration Data
 *
 * Description: This example shows the use of the the Flash Controller APIs
 * to erase and program simulated calibration data to a specific area in memory.
 * Data in this example is programmed to user area of memory. The "fake"
 * calibration data is stored in a RAM array and set using the memset function,
 * however in a real application this buffer would be filled out using a serial
 * interface such as I2C.
 *
 *                MSP432P401
 *             ------------------
 *         /|\|                  |
 *          | |                  |
 *          --|RST               |
 *            |                  |
 *            |                  |
 *            |                  |
 *            |                  |
 *            |                  |
 *
 * Author: Timothy Logan (Modified by John Morrison for E2E post)
 *
 *
 * Description of Modification: The modification assigns a DATA_SECTION in
 * memory called ".mydata" (please reference msp432p401r.cmd which has also been
 * modified for this purpose).  The pragma DATA_SECTION is used to declare that
 * var1 is a variable stored in the memory section, ".mydata".  Then DATA_ALIGN
 * is used to align 2 bytes of memory to that variable since it will be a
 * uint16_t.  The main code will store the value in flash where var1 is stored
 * into the local variable (not stored in flash) var1LOCAL.  var1LOCAL has 1
 * added to it on every power on of the device and then is stored in the first
 * index of the array simulatedCalibrationData[] which is then stored into flash.
 * This updates the var1 in flash.
 ******************************************************************************/
/* DriverLib Includes */
#include <ti/devices/msp432p4xx/driverlib/driverlib.h>

/* Standard Includes */
#include <stdint.h>
#include <stdbool.h>
#include <string.h>

#define CALIBRATION_START 0x0003F000

/* Statics */
uint16_t simulatedCalibrationData[2048];

/* pragmas to define the variables in flash */
#pragma DATA_SECTION(var1, ".mydata");
#pragma DATA_ALIGN(var1, 2);
uint16_t var1, var1LOCAL = 0;

int main(void)
{
    var1LOCAL = var1;
    var1LOCAL++;
    /* Since this program has a huge buffer that simulates the calibration data,
     * halting the watch dog is done in the reset ISR to avoid a watchdog
     * timeout during the zero
     */

    /* Setting our MCLK to 48MHz for faster programming */
    MAP_PCM_setCoreVoltageLevel(PCM_VCORE1);
    FlashCtl_setWaitState(FLASH_BANK0, 2);
    FlashCtl_setWaitState(FLASH_BANK1, 2);
    MAP_CS_setDCOCenteredFrequency(CS_DCO_FREQUENCY_48);

    /* Initializing our buffer to a pattern of 0xA5 */
    memset(simulatedCalibrationData, 0xA5, 4096);
    simulatedCalibrationData[0] = var1LOCAL;

    /* Unprotecting Info Bank 0, Sector 0  */
    MAP_FlashCtl_unprotectSector(FLASH_MAIN_MEMORY_SPACE_BANK1,FLASH_SECTOR31);

    /* Trying to erase the sector. Within this function, the API will
        automatically try to erase the maximum number of tries. If it fails,
         trap in an infinite loop */
    if(!MAP_FlashCtl_eraseSector(CALIBRATION_START))
        while(1);

    /* Trying to program the memory. Within this function, the API will
        automatically try to program the maximum number of tries. If it fails,
        trap inside an infinite loop */
    if(!MAP_FlashCtl_programMemory(simulatedCalibrationData,
            (void*) CALIBRATION_START, 4096))
                while(1);

    /* Setting the sector back to protected  */
    MAP_FlashCtl_protectSector(FLASH_MAIN_MEMORY_SPACE_BANK1,FLASH_SECTOR31);

    /* Going to LPM3 when not in use */
    while (1)
    {
        MAP_PCM_gotoLPM3();
    }
}
